#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import os
import sys

from datetime import datetime


def parse_arguments(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-t', required=True, type=int, dest='timeout',
        help=('Interval to consider when checking health check output '
              'age, in seconds.'))
    parser.add_argument('result_file')
    return parser.parse_args(args)


def main(args=None):
    args = parse_arguments(args)

    timeout = args.timeout
    result_file = args.result_file
    now = datetime.now()

    # Check if file exists
    if not os.path.exists(result_file):
        print("CRITICAL: health check result file not found.")
        sys.exit(2)

    # Check if file is newer than -t seconds
    last_modified = datetime.fromtimestamp(os.path.getmtime(result_file))
    if (now - last_modified).seconds > timeout:
        print("WARNING: health check last run finished more than {} seconds "
              "ago.".format(timeout))
        sys.exit(1)

    # Check if check output is 'OK'
    result_file_contents = '\n'.join(open(result_file).readlines())
    if not 'OK' in result_file_contents:
        print("CRITICAL: health check didn't return OK: ")
        print(result_file_contents)
        sys.exit(2)

    print("OK: tests passed.")
    sys.exit(0)


if __name__ == "__main__":
    main()
