#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import fnmatch
import logging
import os
import subprocess
import tempfile

from distutils import dir_util


def main():
    logging.info("Searching for static directories...")
    # Directory tree traversal code
    # taken from http://stackoverflow.com/a/2186565/729970
    matches = []
    working_dir = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))
    path = os.path.abspath(os.path.join(working_dir, '..', 'webui'))
    for root, dirnames, filenames in os.walk(path):
        for dirname in fnmatch.filter(dirnames, 'static'):
            # skip the top level static dir since where django collects static
            # files.
            skip_path = os.path.join(path, 'static')
            dir_path = os.path.join(root, dirname)

            if dir_path == skip_path:
                continue

            matches.append(dir_path)

    logging.info("matches: %s", matches)

    tmp = tempfile.mkdtemp()
    tmp_path = os.path.join(tmp, 'static')
    for dirname in matches:
        dir_util.copy_tree(dirname, tmp_path, update=True)

    tarfile = '{}/webui_content.tgz'.format(tmp)
    subprocess.check_output(['tar', 'czvf', tarfile, '-C', tmp, 'static'])

    # Change to the temporary directory so swift doesn't add path bits to the
    # filename
    os.chdir(tmp)
    subprocess.check_output(
        ['swift', 'upload', 'charmblobs', os.path.basename(tarfile)]
    )


if __name__ == "__main__":
    logging.basicConfig(level=logging.WARN)
    main()
