# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import shutil
import tempfile
import unittest

import mock


class TestUploadPackage(unittest.TestCase):
    '''Tests for the upload_package module.'''

    subticket_list = [{
        'status': 'New',
        'current_workflow_step': 'New',
        'artifact': [{
            'id': 6,
            'type': 'SPU',
            'name': 'package_0.1_source.changes',
            'reference': 'https://swift.url/package_0.1_source.changes',
            'resource_uri': '/api/v1/fullsubticketartifact/6/'
        }, {
            'id': 5,
            'type': 'SPU',
            'name': 'package_0.1.tar.gz',
            'reference': 'https://swift.url/package_0.1.tar.gz',
            'resource_uri': '/api/v1/fullsubticketartifact/5/'
        }, {
            'id': 4,
            'type': 'SPU',
            'name': 'package_0.1.dsc',
            'reference': 'https://swift.url/package_0.1.dsc',
            'resource_uri': '/api/v1/fullsubticketartifact/4/'
        }],
        'assignee': 'user_id',
        'source_package_upload': {
            'version': '0.1',
            'sourcepackage': {
                'id': 1,
                'name': 'package',
                'resource_uri': '/api/v1/sourcepackage/1/'
            },
            'id': 2,
            'resource_uri': '/api/v1/spu/2/'
        },
        'id': 2,
        'resource_uri': '/api/v1/fullsubticket/2/'
    }]

    def setUp(self):
        super(TestUploadPackage, self).setUp()
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        # we can't import settings unless this is mocked up front
        m = mock.patch('ci_utils.unit_config._unit_config')
        self.addCleanup(m.stop)
        m.start()
        mod = __import__('bsbuilder.upload_package')
        self.upload_package = mod.upload_package

    @mock.patch('bsbuilder.upload_package.create_temp_dir')
    def test_get_source_files(self, temp_dir_mock):
        temp_dir_mock.return_value = self.tmpdir
        mocked_ds = mock.Mock()
        mocked_ds.get_file().return_value = 'foo'
        mocked_ds.get_file.return_value = 'foo'

        file_list = [a['name'] for a in self.subticket_list[0]['artifact']]
        spu = self.subticket_list[0]['source_package_upload']
        source_name = spu['sourcepackage']['name']
        source_version = spu['version']

        results = self.upload_package.get_source_files(
            self.subticket_list, mocked_ds)

        self.assertEqual(1, len(results))
        [result] = results

        self.assertEqual(source_name, result['name'])
        self.assertEqual(source_version, result['version'])
        self.assertEqual('/api/v1/fullsubticket/2/', result['resource'])
        self.assertEqual(self.tmpdir, result['location'])
        self.assertItemsEqual(file_list, result['files'])
        mocked_ds.get_file.assert_called_with('package_0.1.dsc')
