# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import fcntl


RESULT_DIR = "/var/cache/"


class HealthCheck(object):

    def __init__(self, service_name):
        self.service_name = service_name

    def __enter__(self):
        try:
            self.result_file = open(
                os.path.join(
                    RESULT_DIR, '{}.health'.format(self.service_name)),
                'w')
            fcntl.flock(self.result_file, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return self
        except IOError as exc:
            if exc.errno == 11:
                print("Another instance is running.")
                sys.exit(1)
            else:
                raise

    def __exit__(self, exception, value, traceback):
        fcntl.flock(self.result_file, fcntl.LOCK_UN)
        self.result_file.close()

    def write_result_ok(self):
        self.result_file.write("OK")

    def write_result_failed(self, output=None):
        self.result_file.write("FAILED")
        if output is not None:
            self.result_file.write(str(output))
