# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import imp
import os
import sys
import json

from cStringIO import StringIO
from contextlib import contextmanager

from requests import exceptions

from ci_utils.testing import TestCaseWithGnupg


HERE = os.path.abspath(os.path.dirname(__file__))


@contextmanager
def capture_stderr(command, *args, **kwargs):
    out, sys.stderr = sys.stderr, StringIO()
    command(*args, **kwargs)
    sys.stderr.seek(0)
    yield sys.stderr.read()
    sys.stderr = out


@contextmanager
def capture_stdout(command, *args, **kwargs):
    out, sys.stdout = sys.stdout, StringIO()
    command(*args, **kwargs)
    sys.stdout.seek(0)
    yield sys.stdout.read()
    sys.stdout = out


class FakeResponse:
    """Object to mock a python-requests response."""

    def __init__(self, code, content, reason=None):
        self.status_code = code
        self.content = content
        self.reason = reason

    def raise_for_status(self):
        if 400 == self.status_code:
            error = '%s Client Error: %s' % (self.status_code, self.reason)
            e = exceptions.ConnectionError(error)
            raise e
        elif 400 < self.status_code < 500:
            error = '%s Client Error: %s' % (self.status_code, self.reason)
            e = exceptions.HTTPError(error, response=self)
            raise e
        elif 500 <= self.status_code < 600:
            error = '%s Server Error: %s' % (self.status_code, self.reason)
            e = exceptions.HTTPError(error, response=self)
            raise e
        else:
            pass

    def __repr__(self):
        return self.content

    def __iter__(self):
        return (line for line in self.content.split("\n"))

    def iter_content(self, chunk_size):
        return self.__iter__()

    def read(self):
        return self.content

    def json(self):
        return json.loads(self.content)

    def _status_code(self):
        pass


class MainScriptTestCase(TestCaseWithGnupg):
    """Load the 'ubuntu-ci' script module for tests.

    Allows calls to `self.cli.main(args)` and similar.
    """

    def setUp(self):
        super(MainScriptTestCase, self).setUp()
        self.cli = imp.load_source(
            "ubuntu_ci",
            os.path.join(HERE, "../../ubuntu-ci"),
        )
