# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json
import logging
import requests


log = logging.getLogger()


HEADERS = {'content-type': 'application/json'}
API_URL = '/api/v1/'
GK_URL = '/gk/api/v1/'
TICKET_BASE = API_URL + 'ticket/'
SPU_BASE = API_URL + 'spu/'
SOURCEPACKAGE_BASE = API_URL + 'sourcepackage/'
TICKET_ARTIFACT_BASE = API_URL + 'ticketartifact/'
SUBTICKET_ARTIFACT_BASE = API_URL + 'subticketartifact/'
SUBTICKET_BASE = API_URL + 'subticket/'
TICKET_STATUS_BASE = API_URL + 'ticketstatus/'
HOME = os.environ["HOME"]

CI_URL = ""


class InputError(Exception):
    """Exception raised for errors in the input."""


def post(url, data, headers=HEADERS, patch=False):
    response = requests.post(url=url, data=json.dumps(data), headers=HEADERS)
    response.raise_for_status()
    headers = response.headers
    return headers['location']


def patch(url, data, headers=HEADERS):
    response = requests.patch(url=url, data=json.dumps(data), headers=HEADERS)
    response.raise_for_status()


def get(url):
    response = requests.get(url)
    response.raise_for_status()
    return response.json()


def parse_id(location):
    try:
        id = location.split('/')[6]
        int(id)
    except ValueError, exc:
        raise exc
    return id


def get_sourcepackage_uri(sourcepackage):
    url = CI_URL + SOURCEPACKAGE_BASE + '?name__iexact=' + sourcepackage
    data = get(url)
    try:
        sourcepackage_uri = data['objects'][0]['resource_uri']
        return sourcepackage_uri
    except IndexError:
        # Package not found.
        log.info("Source package not found: %s" % sourcepackage)
        return ''


def assert_valid_package_list(package_list_str):
    if package_list_str and " " in package_list_str:
        raise InputError("Added and removed packages have to be "
                         "comma-separated lists with no spaces.")
