# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import yaml

BASEDIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

DEBUG = False
TEMPLATE_DEBUG = DEBUG
CONFIGURATION = 'devel'

ALLOWED_HOSTS = ['*']

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)
MANAGERS = ADMINS


def _unit_config():
    path = os.path.join(BASEDIR, '../../../unit_config')
    config = {}
    try:
        with open(path) as f:
            config = yaml.safe_load(f.read())
    except:
        print('Unable to use unit_config(%s), defaulting values' % path)
    return config
_cfg = _unit_config()

LAUNCHPAD_USER = _cfg.get('launchpad_user', None)
LAUNCHPAD_PPA_OWNER = _cfg.get('launchpad_ppa_owner', None)
LAUNCHPAD_API_BASE = _cfg.get(
    'launchpad_api_base', 'https://api.launchpad.net/1.0')
OAUTH_CONSUMER_KEY = _cfg.get('oauth_consumer_key', None)
OAUTH_TOKEN = _cfg.get('oauth_token', None)
OAUTH_TOKEN_SECRET = _cfg.get('oauth_token_secret', None)
OAUTH_REALM = _cfg.get('oauth_realm', 'https://api.launchpad.net/')
PPA_PATTERN = _cfg.get('ppa_pattern', r'ci-pool-\d+')
PRIVATE_PPAS_ONLY = _cfg.get('private_ppas_only', None)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASEDIR, 'db.sqlite'),
    },
}

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = False

TIME_ZONE = 'UTC'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

ROOT_URLCONF = 'ppa_assigner.urls'

LOCAL_APPS = (
    'ppa_assigner',
)

TASTYPIE_DEFAULT_FORMATS = ['json', 'jsonp']

REQUIRED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'south',
    'tastypie',   # needed for ApiKeyAuthentication
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler',
            'filters': ['require_debug_false'],
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

# By default the API_TOKEN is disabled.  Set it to a strong key
# to enable API write access.
API_TOKEN = 'DISABLED'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'zlv5#5_@*5re)(s$0&r98i*5*g6k=x!$0lo)fl!+^2l6#)5hqb'

# Use local settings if available
try:
    p = os.path.join(BASEDIR, 'local_settings.py')
    if os.path.exists(p):
        execfile(p)
except ImportError:
    pass

INSTALLED_APPS = LOCAL_APPS + REQUIRED_APPS

# files created by restish juju charm
pgsql = os.path.join(BASEDIR, '../../../pgsql.json')
if os.path.exists(pgsql):
    print('pgsql settings from %s' % pgsql)
    pgsql = json.load(open(pgsql))
    DATABASES['default'] = {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': pgsql['database'],
        'USER': pgsql['schema_user'],
        'PASSWORD': pgsql['schema_password'],
        'HOST': pgsql['host'],
        'PORT': pgsql['port'],
    }

allowed = os.path.join(BASEDIR, '../../../allowed_hosts.json')
if os.path.exists(allowed):
    ALLOWED_HOSTS = json.load(open(allowed))
    print('ALLOWED_HOSTS from (%s): %s' % (allowed, ', '.join(ALLOWED_HOSTS)))

logging = os.path.join(BASEDIR, '../../../logging.json')
if os.path.exists(logging):
    data = {}
    with open(logging) as f:
        data = json.load(f)

    log_dir = data.get('log_dir')
    log_level = data.get('log_level')

    if log_dir and os.path.exists(log_dir) and log_level:
        log_file = os.path.join(log_dir, 'django.log')
        LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': {
                'standard': {
                    'format': (
                        "[%(asctime)s] %(levelname)s [%(name)s"
                        ":%(lineno)s] %(message)s"),
                    'datefmt': "%d/%b/%Y %H:%M:%S"
                },
            },
            'handlers': {
                'file': {
                    'level': log_level,
                    'class': 'logging.FileHandler',
                    'filename': log_file,
                    'formatter': 'standard',
                },
            },
            'loggers': {
                'django': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
                '': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
            },
        }
