# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import shutil
import tempfile


SUPPORTED_SERIES = [
    'precise',
    'trusty',
    'utopic',
]


SUPPORTED_POCKETS = {
    '': 'Release',
    # '-updates': 'Updates',
}


def valid_suites():
    suites = []
    for pocket in SUPPORTED_POCKETS.keys():
        for series in SUPPORTED_SERIES:
            suites.append('{}{}'.format(series, pocket))
    return suites


class TmpDir(object):
    '''Create a temporary directory with a context manager'''

    def __init__(self, suffix='', prefix='tmp', dir=None):
        self.path = tempfile.mkdtemp(suffix, prefix, dir)

    def __enter__(self):
        return self.path

    def __exit__(self, exc, value, tb):
        shutil.rmtree(self.path)
