#!/usr/bin/env python3
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging
import json

from subunitresults.utils import retry
from subunitresults.ci_utils import amqp_utils, amqp_worker


logging.basicConfig(level='INFO')
logger = logging.getLogger(__name__)


def get_item(queue):
    return queue.get(block=True, timeout=None)


def get_subunit_path(json_blob):
    d = json.loads(json_blob)  # XXX what should we do here if json is invalid?
    try:
        subunit_path = d['subunit_path']
    except KeyError:
        # XXX what should we do here in case of bad message?
        raise Exception("subunit_path not found in json message")
    return subunit_path


@retry(Exception, tries=5, delay=1, backoff=2, logger=None)
def fetch_subunit(data_store, url):
    contents = data_store.get_file(url)
    return contents


def put_attachment(data_store, file_name, contents, content_type):
    file_path = data_store.put_file(file_name, contents, content_type)
    return file_path


class Worker:
    def __init__(self, data_store, queue):
        self.queue = queue
        self.data_store = data_store

    def run(self):
        while True:
            self.do_one_job()

    def do_one_job(self):
        pass
        # json_blob = get_item(self.queue)
        # path = get_subunit_path(json_blob)
        # subunit_file = fetch_subunit(data_store, self.path)
        # parse subunit here
        # send all attachments to Swift


class RabbitWorker(amqp_worker.AMQPWorker):
    def __init__(self, logger_name, data_store, queue):
        super().__init__(logger_name)
        self.logger_name
        self.data_store
        self.queue

    def handle_request(self, params, logger):
        worker = Worker(self.data_store, self.queue)
        try:
            worker.do_one_job()
            return (amqp_utils.progress_completed, {})
        except Exception:
            return (amqp_utils.progress_failed, {})
