#!/usr/bin/env python3
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
from queue import Queue
import unittest

from testtools.content_type import ContentType
from ucitests import fixtures as uci_fixtures

from ci_utils.testing import fixtures
from subunitresults import queue_worker


class TestQueueWorker(unittest.TestCase):

    def test_get_item(self):
        test_msg = 'hello'
        q = Queue()
        q.put(test_msg)
        msg = queue_worker.get_item(q)
        self.assertEqual(msg, test_msg)

    def test_get_subunit_path(self):
        test_path = 'http://foo/12345/results.sub'
        json_blob = json.dumps({
            'subunit_path': '%s' % test_path,
            'ticket_id': 12345
        })
        path = queue_worker.get_subunit_path(json_blob)
        self.assertEqual(path, test_path)

    def test_fetch_subunit(self):
        uci_fixtures.set_uniq_cwd(self)
        ds_path = os.path.join(self.uniq_dir, 'data_store')
        ds = fixtures.FakeDataStore(ds_path, None)
        self.addCleanup(ds.delete, recursive=True)
        filename = 'test_file'
        test_path = 'http://foo/12345/results.sub'
        json_blob = json.dumps({
            'subunit_path': '%s' % test_path,
            'ticket_id': 12345
        }).encode('utf-8')
        content_type = ContentType('application', 'json')
        ds.put_file(filename, json_blob, content_type)
        self.addCleanup(ds.delete_file, filename)
        fetched = queue_worker.fetch_subunit(ds, filename)
        self.assertEqual(json_blob, fetched)

    def test_put_attachment(self):
        uci_fixtures.set_uniq_cwd(self)
        ds_path = os.path.join(self.uniq_dir, 'data_store')
        ds = fixtures.FakeDataStore(ds_path, None)
        self.addCleanup(ds.delete, recursive=True)
        filename = 'test_file'
        contents = b'{"subunit_path": "http://example/foo"}'
        content_type = ContentType('application', 'json')
        queue_worker.put_attachment(ds, filename, contents, content_type)
        self.addCleanup(ds.delete_file, filename)
        self.assertEqual([filename], ds.list_files())

    # def test_runner(self):
    #    q = Queue()
    #    q.put('{"subunit_path": "http://example/foo"}')
    #    runner = queue_worker.Runner(q)
    #    runner.do_one_job()
